@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2015-2016  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2015-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -48,7 +48,7 @@ struct test_handshake_state {
 static void test_handshake_state_free(struct handshake_state *hs)
 {
 	struct test_handshake_state *ths =
-			container_of(hs, struct test_handshake_state, super);
+			l_container_of(hs, struct test_handshake_state, super);
 
 	l_free(ths);
 }
@@ -1989,7 +1989,7 @@ static void wsc_test_pbc_handshake(const void *data)
 	char *hex;
 	struct l_settings *settings;
 
-	eap_init(1400);
+	eap_init();
 	eapol_init();
 
 	hs = test_handshake_state_new(1);
@@ -2097,7 +2097,7 @@ static void wsc_test_retransmission_no_fragmentation(const void *data)
 	char *hex;
 	struct l_settings *settings;
 
-	eap_init(1400);
+	eap_init();
 	eapol_init();
 
 	hs = test_handshake_state_new(1);
